/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Logging.
 * @author Christian Grothoff
 * @file util/logging.c
 **/
#include "util/logging.h"
#include "util/storage.h"
#include "util/xmalloc.h"

static FILE * logfile = NULL;
static int loglevel = 0;

void initLogging(char * logfilename,
		 int level) {
  loglevel = level;
  if (logfilename != NULL) {
    logfilename = expandFileName(logfilename);
    logfile = fopen(logfilename, "a+");
    xfree(logfilename, "initLogging - logfilename");
  }
}

void initLoggingFile(FILE * f,
		     int level) {
  loglevel = level;
  logfile = f;
}

void setLogLevel(int i) {
  loglevel = i;
}

void doneLogging() {
  if (logfile != NULL)
    fclose(logfile);
  logfile = NULL;
  loglevel = 0;
}

/**
 * Log a debug message
 * @param format the string describing the error message
 **/
void print(const char *format, ...) {
  va_list	args;  

  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * Errexit - log an error message and exit.
 * Note that this method explicitly also writes
 * an null-termination at the end of the message
 * to signal the waiting parent that the child is
 * aborting.
 * @param format the string describing the error message
 **/
void errexit(const char *format, ...) {
  va_list args;

  if (loglevel < LOG_FATAL)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
  exit(1);
}

/**
 * @param format the string describing the error message
 **/
void info__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_INFO)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * @param format the string describing the error message
 **/
void message__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_MESSAGE)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * @param format the string describing the error message
 **/
void warning__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_WARNING)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * @param format the string describing the error message
 **/
void error__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_ERROR)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * @param format the string describing the error message
 **/
void debug__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_DEBUG)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/**
 * @param format the string describing the error message
 **/
void failure__(const char *format, ...) {
  va_list	args;  

  if (loglevel < LOG_FAILURE)
    return;
  va_start(args, format);
  if (logfile != NULL) {
    vfprintf(logfile, format, args);
    fflush(logfile);
  } else
    vfprintf(stderr, format, args);
  va_end(args);
}

/* end of logging.c */
